function H = calculateEntropyImage(I,patchSize, bins, weighting, ...
    normalization, estimation, entropyMaes)

% Calculation of entropy image for 2D image
%
% - patchSize must be an odd number
% - special weighting only for patchSize 7
%
% I: input image
% patchSize: size of patch
% bins: number of bins
% weighting: spatial weighting {'gaussian','special','identity'}
% normalization: 'local' or 'global' normalization
% estmation: density estimation {'parzen','histogramming'}
% entropyMaes: entropy measure {'shannon', 'burg', 'renyi'}
%
% Author: Christian Wachinger, CAMP, TU Mnchen, 2010
%
% References:
%  C. Wachinger, N. Navab
%  Structural Image Representation for Image Registration
%  CVPR Workshops, Mathematical Methods in Biomedical Image Analysis (MMBIA),
%  2010. 
%
%  C. Wachinger, N. Navab
%  Entropy and Laplacian Images: Structural Representations for Multi-Modal Registration
%  Medical Image Analysis, 2011.
%

if (nargin < 2) patchSize = 11; end
if (nargin < 3) bins = 16; end
if (nargin < 4) weighting = 'gaussian'; end
if (nargin < 5) normalization = 'global'; end
if (nargin < 6) estimation = 'parzen'; end
if (nargin < 7) entropyMaes = 'shannon'; end


% Set mask for spatial weighting for density estimation
if (strcmp(weighting,'gaussian'))
    gaussKernel = fspecial('gaussian', [patchSize patchSize],4);
    disp('Gaussian weighting')    
elseif (strcmp(weighting,'special'))
    gaussKernel =[
    0.0146    0.0167    0.0184    0.0199    0.0182    0.0165    0.0144;
    0.0169    0.0218    0.0218    0.0231    0.0224    0.0204    0.0163;
    0.0186    0.0230    0.0243    0.0249    0.0243    0.0216    0.0180;
    0.0195    0.0231    0.0247    0.0259    0.0253    0.0227    0.0198;
    0.0188    0.0228    0.0243    0.0255    0.0243    0.0214    0.0178;
    0.0171    0.0200    0.0220    0.0225    0.0226    0.0206    0.0161;
    0.0148    0.0157    0.0174    0.0196    0.0176    0.0159    0.0142];
    patchSize = 7;
    disp('Special weighting')
else
    gaussKernel = ones(patchSize, patchSize);
    disp('Identity weighting')
end

H = zeros(size(I));
halfPatch = (patchSize-1)/2;

% Global normalization
if (strcmp(normalization,'global'))
    I = double(I);
    I = I - min(I(:)) + 0.0001;
    I = ceil( I / max(I(:)) * bins);
end


for i = 1+halfPatch:size(I,1)-halfPatch
    for j = 1+halfPatch:size(I,2)-halfPatch
        H(i,j) = entropyEstimation(I(i-halfPatch:i+halfPatch,j-halfPatch:j+halfPatch),... 
            gaussKernel, bins, normalization, estimation, entropyMaes);
    end
end

